package com.gic.auth.service.outer.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.application.center.api.service.AuthorizeService;
import com.gic.auth.dto.*;
import com.gic.auth.entity.TabSysResourceGroup;
import com.gic.auth.service.*;
import com.gic.auth.utils.resourcegroup.ResourceGroupUtils;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.exception.CommonException;
import com.gic.enterprise.service.UnionEnterpriseAuthApiService;
import com.gic.mq.sdk.GicMQClient;
import com.gic.store.service.StoreWidgetApiService;

/**
 * 资源组
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/1/8 4:38 PM 
 */
@Service("resourceGroupApiService")
public class ResourceGroupApiServiceImpl implements ResourceGroupApiService{
    private static Logger logger = LogManager.getLogger(ResourceGroupApiServiceImpl.class);
    @Autowired
    private ResourceGroupService resourceGroupService;
    @Autowired
    private AuthorizeService authorizeService;
    @Autowired
    private UserResourceService userResourceService;
    @Autowired
    private UserService userService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private UnionEnterpriseAuthApiService unionEnterpriseAuthApiService;
    @Autowired
    private AuthCodeApiService authCodeApiService;

    @Override
    public ServiceResponse<Integer> saveResourceGroup(ResourceGroupDTO resourceGroup) {
        String validParamResponse = validParam(resourceGroup);
        if (validParamResponse != null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), validParamResponse);
        }
        Integer resourceGroupId = resourceGroupService.saveResourceGroup(resourceGroup);
        return ServiceResponse.success(resourceGroupId);
    }

    @Override
    public ServiceResponse<String> editResourceGroup(ResourceGroupDTO resourceGroup) {
        String validParamResponse = validParam(resourceGroup);
        if (validParamResponse != null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), validParamResponse);
        }

        //是否联合授权，如果是
        UnionEnterpriseAuthDTO unionEnterpriseAuthDTO = unionEnterpriseAuthApiService
                .getUnionEnterpriseAuthByResourceGroupId(resourceGroup.getResourceGroupId()).getResult();
        if (unionEnterpriseAuthDTO != null) {
            ServiceResponse authCodeResult = authCodeApiService.validateAuthCode(resourceGroup.getAuthCodeId(), resourceGroup.getAuthCode());
            if (!authCodeResult.isSuccess()) {
                return authCodeResult;
            }
            ResourceGroupDTO oldResourceGroup = EntityUtil.changeEntityNew(ResourceGroupDTO.class,
                    checkExistResourceGroup(resourceGroup.getResourceGroupId()));
            //通知业务方数据同步联合调整
            syncResourceAuthChangeMQ(unionEnterpriseAuthDTO.getOwnEnterpriseId(), unionEnterpriseAuthDTO.getUnionEnterpriseId(),
                    resourceGroup, oldResourceGroup);
        }

        resourceGroupService.editResourceGroup(resourceGroup);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> syncResourceAuthChangeMQ(Integer ownEnterpriseId, Integer unionEnterpriseId,
                                                          ResourceGroupDTO newResourceGroup, ResourceGroupDTO oldResourceGroup) {
        UnionEnterpriseAuthResNoticeDTO unionEnterpriseAuthResNoticeDTO = new UnionEnterpriseAuthResNoticeDTO();
        unionEnterpriseAuthResNoticeDTO.setGoodsResourceId(newResourceGroup.getGoodsResourceId());
        unionEnterpriseAuthResNoticeDTO.setStoreResource(newResourceGroup.getStoreResource());
        unionEnterpriseAuthResNoticeDTO.setOwnEnterpriseId(ownEnterpriseId);
        unionEnterpriseAuthResNoticeDTO.setUnionEnterpriseId(unionEnterpriseId);

        CustomUserAreaDTO customUserAreaDTO = ResourceGroupUtils
                .getResource(ResourceGroupUtils.CUSTOM_RESOURCE, EntityUtil.changeEntityNew(TabSysResourceGroup.class, newResourceGroup));
        unionEnterpriseAuthResNoticeDTO.setCustomAreaDTO(EntityUtil.changeEntityNew(CustomAreaDTO.class, customUserAreaDTO));
        List<UserResourceChangeInfoDTO> userResourceInfoList = new ArrayList<>();


        Map<Integer, List<Long>> oldUserAreaMap = dealUserResourceData(oldResourceGroup);

        if (newResourceGroup.getCustomUser() == 1) {
            Map<Integer, List<Long>> newUserAreaMap = dealUserResourceData(newResourceGroup);
            newUserAreaMap.entrySet().forEach(e -> {
                List<Long> oldAreaList = oldUserAreaMap.get(e.getKey());

                UserResourceChangeInfoDTO userResourceChangeInfoDTO = new UserResourceChangeInfoDTO();
                userResourceChangeInfoDTO.setType(e.getKey());
                if (oldAreaList == null) {
                    userResourceChangeInfoDTO.setAddList(e.getValue());
                } else {
                    List<Long> newAreaList = e.getValue();

                    List<Long> addList = new ArrayList<>();
                    List<Long> deleteList = new ArrayList<>();

                    for (Long newArea : newAreaList) {
                        if (!oldAreaList.contains(newArea)) {
                            //如果新的数据在旧的数据里面不存在了，则说明增加了域
                            addList.add(newArea);
                        }
                    }
                    for (Long oldArea : oldAreaList) {
                        if (!newAreaList.contains(oldArea)) {
                            //如果旧的数据在新的数据里面不存在，则说明被删除了域
                            deleteList.add(oldArea);
                        }
                    }
                    userResourceChangeInfoDTO.setAddList(addList);
                    userResourceChangeInfoDTO.setDeleteList(deleteList);
                }

                userResourceInfoList.add(userResourceChangeInfoDTO);
            });
        } else {
            oldUserAreaMap.entrySet().forEach(e -> {
                UserResourceChangeInfoDTO userResourceChangeInfoDTO = new UserResourceChangeInfoDTO();
                userResourceChangeInfoDTO.setType(e.getKey());
                userResourceChangeInfoDTO.setDeleteList(e.getValue());

                userResourceInfoList.add(userResourceChangeInfoDTO);
            });
        }

        unionEnterpriseAuthResNoticeDTO.setUserResourceInfoList(userResourceInfoList);

        GicMQClient client = GICMQClientUtil.getClientInstance();
        Map<String, Object> params = new HashMap<>(4);
        params.put("unionEnterpriseAuthResNoticeDTO", unionEnterpriseAuthResNoticeDTO);
        try {
            logger.info("联合商户资源授权通知");
            client.sendMessage("syncResourceAuthChangeMQ", JSONObject.toJSONString(params));
        } catch (Exception e) {
            logger.warn("联合商户资源授权路由:{}", e.getMessage(), e);
        }

        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<ResourceGroupDTO>> pageResourceGroup(String name, Integer enterpriseId,
                                                                     Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page<TabSysResourceGroup> page = resourceGroupService.pageResourceGroup(name, enterpriseId, pageNum, pageSize);
        Page<ResourceGroupDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, ResourceGroupDTO.class);
        List<ResourceGroupDTO> resourceList = resultPage.getResult();
        if (CollectionUtils.isNotEmpty(resourceList)) {
            //查询授权管理员数量
            List<Integer> resourceIds = resourceList.stream()
                    .map(ResourceGroupDTO::getResourceGroupId)
                    .collect(Collectors.toList());
            Map<Integer, Integer> map = userResourceService.countByResourceIds(enterpriseId, resourceIds);
            for (ResourceGroupDTO resource : resourceList) {
                Integer count = map.get(resource.getResourceGroupId());
                resource.setUserResourceCount(count == null ? 0 : count);
            }
        }
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<List<ResourceGroupDTO>> listResourceGroup(String name, Integer enterpriseId) {
        return ServiceResponse.success(EntityUtil.changeEntityListNew(ResourceGroupDTO.class,
                resourceGroupService.listResourceGroup(name, enterpriseId)));
    }

    @Override
    public ServiceResponse<ResourceGroupDTO> getResourceGroup(Integer resourceGroupId) {
        TabSysResourceGroup record = resourceGroupService.getResourceGroup(resourceGroupId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源组不存在");
        }
        ResourceGroupDTO result = EntityUtil.changeEntityNew(ResourceGroupDTO.class,
                record);
        if (StringUtils.isNotBlank(result.getCustomUserData())) {
            result.setCustomUserList(JSON.parseArray(result.getCustomUserData(), ResourceGroupCustomUserDTO.class));
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<String> deleteResourceGroup(Integer resourceGroupId) {
        TabSysResourceGroup record = checkExistResourceGroup(resourceGroupId);
        //是否联合授权，如果是，则不允许删除
        if (unionEnterpriseAuthApiService.hasUnionEnterpriseAuth(resourceGroupId).getResult()) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "该资源组已授权给其他商户，不能删除");
        }
        //验证是否有管理员使用中
        boolean exist = this.userResourceService.existUserResource(record.getEnterpriseId(), record.getResourceGroupId());
        if (exist) {
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "该资源组正在使用中，不能删除");
        }
        resourceGroupService.deleteResourceGroup(resourceGroupId);
        return ServiceResponse.success(record.getResourceGroupName());
    }

    @Override
    public ServiceResponse<ResourceGroupDataDTO> getResourceGroupByUserId(Integer userId) {
        return ServiceResponse.success(ResourceGroupUtils.getResource(userId));
    }

    @Override
    public ServiceResponse<StoreResourceDTO> getStoreResourceByUserId(Integer userId) {
        return ServiceResponse.success(ResourceGroupUtils.getResource(userId, ResourceGroupUtils.STORE_RESOURCE));
    }

    @Override
    public ServiceResponse<CustomUserAreaDTO> getCustomUserByUserId(Integer userId) {
        return ServiceResponse.success(ResourceGroupUtils.getResource(userId, ResourceGroupUtils.CUSTOM_RESOURCE));
    }

    /**
     * 参数验证
     * @param resourceGroup
     * @return null：验证OK
     */
    private String validParam(ResourceGroupDTO resourceGroup) {
        Integer resourceGroupId = resourceGroup.getResourceGroupId();
        if (resourceGroupId != null) {
            TabSysResourceGroup record = resourceGroupService.getResourceGroup(resourceGroupId);
            if (record == null) {
                return"资源组不存在";
            }
        }
        //param valid
        if (StringUtils.isBlank(resourceGroup.getResourceGroupName())) {
            return "资源组名称不能为空";
        }
        Integer enterpriseId =  resourceGroup.getEnterpriseId();
        if (enterpriseId == null) {
            return "商户ID不能为空";
        }
        if (resourceGroupService.isRepeatResourceGroupName(resourceGroupId, resourceGroup.getResourceGroupName(), enterpriseId)) {
            return "资源组名称重复";
        }
        Integer customUser = resourceGroup.getCustomUser();
        if (customUser == null) {
            return "用户域授予标志不能为空";
        }
        //用户域是否授予
        boolean hasAuthCustomUser = customUser == 1;
        String customUserData = resourceGroup.getCustomUserData();
        if (hasAuthCustomUser) {
            if (StringUtils.isBlank(customUserData)) {
                return "用户域授予资源为空，请选择不授予";
            }

            try {
                List<ResourceGroupCustomUserDTO> customUserList = JSON.parseArray(customUserData, ResourceGroupCustomUserDTO.class);
            } catch (Exception e) {
                return "用户域数据格式有误";
            }

        }
        return null;
    }

    private TabSysResourceGroup checkExistResourceGroup(Integer resourceGroupId) {
        TabSysResourceGroup record = resourceGroupService.getResourceGroup(resourceGroupId);
        if (record == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "资源组不存在");
        }
        return record;
    }

    private Map<Integer, List<Long>> dealUserResourceData(ResourceGroupDTO oldResourceGroup) {
        //处理旧的用户域数据，用于比对区分增加/删除域数据
        Map<Integer, List<Long>> oldUserAreaMap = new HashMap<>();
        if (oldResourceGroup != null) {
            if (oldResourceGroup.getCustomUser() == 1) {
                List<ResourceGroupCustomUserDTO> customUserList = JSON.parseArray(oldResourceGroup.getCustomUserData(), ResourceGroupCustomUserDTO.class);
                Map<Integer, List<ResourceGroupCustomUserDTO>> map = customUserList.stream().collect(Collectors.groupingBy(ResourceGroupCustomUserDTO::getType));
                map.entrySet().forEach(e -> {
                    List<ResourceGroupCustomUserDTO> tempList = e.getValue();
                    List<Long> areaValueList = new ArrayList<>();
                    for (ResourceGroupCustomUserDTO temp : tempList) {
                        String[] areaValueArr = temp.getAreaValue().split(",");
                        for (String areaValue : areaValueArr) {
                            if (org.apache.commons.lang3.StringUtils.isBlank(areaValue)) {
                                continue;
                            }
                            areaValueList.add(Long.valueOf(areaValue));
                        }
                    }
                    oldUserAreaMap.put(e.getKey(), areaValueList);
                });
            }
        }
        return oldUserAreaMap;
    }
}
